// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "AnimationController.hpp"
#include "game/DawnGame.hpp"

using namespace Dawn;

AnimationController::AnimationController(SceneItem *item) :
  SceneItemComponent(item)
{

}

void AnimationController::addAnimation(Animation *animation) {
  assertNotNull(animation);
  this->animations.push_back(animation);
}

void AnimationController::onSceneUpdate() {
  auto it = this->animations.begin();
  while(it != this->animations.end()) {
    (*it)->tick(this->getGame()->timeManager.delta);
    ++it;
  }
}

void AnimationController::onStart() {
  SceneItemComponent::onStart();
  getScene()->eventSceneUnpausedUpdate.addListener(this, &AnimationController::onSceneUpdate);
}

void AnimationController::onDispose() {
  getScene()->eventSceneUnpausedUpdate.removeListener(this, &AnimationController::onSceneUpdate);
}