// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "dawnsharedlibs.hpp"

namespace Dawn {
  struct ClassGenInfo {
    std::vector<std::string> includes;
    std::string clazz = "Unknown";
    std::string extend = "";
    std::string constructorArgs = "";
    std::string extendArgs = "";

    std::vector<std::string> constructorCode;

    std::vector<std::string> protectedCode;
    std::vector<std::string> protectedProperties;

    std::vector<std::string> publicCode;
    std::vector<std::string> publicProperties;
  };

  struct MethodGenInfo {
    std::string name;
    std::string type = "void";
    std::vector<std::string> body;
    std::string args = "";
    bool_t isStatic = false;
    bool_t isOverride = false;
  };
  
  class CodeGen {
    protected:
      static void line(
        std::vector<std::string> *out,
        std::string contents,
        std::string tabs
      ) {
        out->push_back(tabs + contents);
      }

      static void lines(
        std::vector<std::string> *out,
        std::vector<std::string> lines,
        std::string tabs
      ) {
        auto itLine = lines.begin();
        while(itLine != lines.end()) {
          line(out, *itLine, tabs);
          ++itLine;
        }
      }

      static void classGen(
        std::vector<std::string> *out,
        struct ClassGenInfo info
      ) {
        std::vector<std::string> buffer;

        line(out, "#pragma once", "");
        line(out, "", "");
        if(info.includes.size() > 0) {
          lines(out, info.includes, "");
          line(out, "", "");
        }
        line(out, "namespace Dawn {", "");
          line(out, "class " + info.clazz + (info.extend.size() == 0 ? "{" : " : public " + info.extend + " {" ), "  ");
          if(info.protectedCode.size() > 0) {
            line(out, "protected:", "    ");
              lines(out, info.protectedProperties, "      ");
              line(out, "", "      ");
              lines(out, info.protectedCode, "      ");
          }

          if(info.publicCode.size() > 0 || info.constructorArgs.size() > 0 || info.constructorCode.size() > 0) {
            line(out, "public:", "    ");
              lines(out, info.publicProperties, "      ");
              line(out, "", "      ");
              line(out, info.clazz + "(" + info.constructorArgs + ")" + (info.extend.size() > 0 ? " : " + info.extend + "(" + info.extendArgs + ")" : "") + " {", "      ");
              lines(out, info.constructorCode, "        ");
              line(out, "}",  "      ");
              if(info.publicCode.size() > 0) {
                line(out, "", "      ");
                lines(out, info.publicCode, "      ");
              }
          }
          line(out, "};", "  ");
        line(out, "}", "");
      }

      static void methodGen(
        std::vector<std::string> *out,
        struct MethodGenInfo info
      ) {
        line(out, info.type + " " + info.name + "(" + info.args + ") " + ( info.isOverride ? "override" : "" ) + "{", "");
        lines(out, info.body, "  ");
        line(out, "}", "");
      }
  };
}